//
//  obt@f[^NX
//
//  2002/ 8/19 ˁ@P
//

#include "classes.h"

D2CBuffer::D2CBuffer() {

	m_bfBuffer=NULL;
	m_csStr=NULL;

	clear();

}
D2CBuffer::D2CBuffer(const BUF bfArg,I32 i32Arg) {

	m_bfBuffer=NULL;
	m_csStr=NULL;

	clear();

	assignBuffer(bfArg,i32Arg);

}

D2CBuffer::D2CBuffer(const D2IBufferReader *poArg) {

	m_bfBuffer=NULL;
	m_csStr=NULL;

	clear();

	assignBuffer(poArg);

}

void * D2CBuffer::getInterface(UI32 u32Arg) const {

	switch (u32Arg) {

	case ID_D2IObject:

		return (D2IObject *)this;

	case ID_D2IBufferReader:

		return (D2IBufferReader *)this;

	case ID_D2IBufferWriter:

		return (D2IBufferWriter *)this;

	case ID_D2IBufferedObject:

		return (D2IBufferedObject *)this;

	default:

		return NULL;

	}

};

bool D2CBuffer::assignBuffer(const BUF bfArg,I32 i32Arg) {

	if (bfArg==NULL || i32Arg<1)
		return false;

	clear();

	if (!resizeBuffer(i32Arg))
		return false;

	memcpy(getBuffer(),bfArg,i32Arg);

	return true;

}

bool D2CBuffer::assignBuffer(const D2IBufferReader *poArg) {

	if (poArg==NULL)
		return false;

	BUF bfTmp=poArg->copyBuffer();

	if (bfTmp==NULL)
		return false;

	assignBuffer(bfTmp,poArg->getBufferSize());

	delete [] bfTmp;

	return true;

}

bool D2CBuffer::equalBuffer(const BUF bfArg,I32 i32Arg) const {

	if (bfArg==NULL || getBufferSize()!=i32Arg)
		return false;

	if (getBufferSize()==0)
		return true;

	for (int i=0;i<i32Arg;i++)
		if (readBufferByte(i)!=bfArg[i])
			return false;

	return true;

}

bool D2CBuffer::equalBuffer(const D2IBufferReader *pbrArg) const {

	if (pbrArg==NULL)
		return false;

	if (getBufferSize()==0)
		return true;

	for (int i=0;i<getBufferSize();i++)
		if (readBufferByte(i)!=pbrArg->readBufferByte(i))
			return false;

	return true;

}

bool D2CBuffer::clear() {

	deleteBuffer();
	setBufferSize(0);

	return true;

}

bool D2CBuffer::resizeBuffer(I32 i32Arg,bool bArg) {

	if (i32Arg<0)
		return false;

	if (i32Arg==0) {

		clear();
		return true;

	}

	BUF bfTmp=new UI8[i32Arg];

	if (bfTmp==NULL)
		return false;

	I32 i32Cpy=i32Arg;

	if (i32Cpy>getBufferSize())
		i32Cpy=getBufferSize();

	memcpy(bfTmp,getBuffer(),i32Cpy);

	deleteBuffer();

	setBuffer(bfTmp);
	setBufferSize(i32Arg);

	if (bArg && i32Arg>i32Cpy)
		fillBuffer(0,i32Cpy,i32Arg-1);

	return true;

}

UI32 D2CBuffer::getObjectType() const {

	UI32 u32Tmp=getVersion();
	u32Tmp |=getDataType() << 8;
	u32Tmp |=getEncoding() << 16;

	return u32Tmp;

}

UI8 D2CBuffer::getVersion() const { return VER_D2_0; }
UI8 D2CBuffer::getDataType() const { return TYPE_BUF; }
UI16 D2CBuffer::getEncoding() const { return ENCODE_NONE; }

BUF D2CBuffer::copyBuffer() const {
	
	BUF bfTmp=new UI8[getBufferSize()];

	if (bfTmp==NULL)
		return NULL;

	memcpy(bfTmp,getBuffer(),getBufferSize());

	return bfTmp;

}

BUF D2CBuffer::getBuffer() const { return m_bfBuffer; }

I32 D2CBuffer::getBufferSize() const { return m_i32BufferSize; }

bool D2CBuffer::setBuffer(const BUF bfArg) {

	m_bfBuffer=bfArg;

	return true;

}

bool D2CBuffer::setBufferSize(I32 i32Arg) {

	m_i32BufferSize=i32Arg;

	return true;

}

bool D2CBuffer::deleteBuffer() {

	if (m_bfBuffer==NULL)
		return false;

	delete [] m_bfBuffer;

	return true;

}

CSTR D2CBuffer::getCSTR() {

	if (m_csStr!=NULL)
		delete[] m_csStr;

	m_csStr=new CHR[getBufferSize()+1];

	memcpy(m_csStr,getBuffer(),getBufferSize());
	m_csStr[getBufferSize()]=0;

	return m_csStr;

};

UI8 D2CBuffer::readBufferByte(I32 i32Arg) const {

	if (i32Arg<0 || i32Arg>=getBufferSize())
		return 0;

	return getBuffer()[i32Arg];

}

bool D2CBuffer::writeBuffer(const BUF bfArg,I32 i32Arg1,I32 i32Arg2) {

	if (bfArg==NULL || i32Arg1<0)
		return false;

	if (getBufferSize()<i32Arg1+i32Arg2)
		resizeBuffer(i32Arg1+i32Arg2);

	memcpy(getBuffer()+i32Arg1,bfArg,i32Arg2);

	return true;

}

bool D2CBuffer::writeBufferByte(UI8 u8Arg,I32 i32Arg) {

	if (i32Arg<0)
		return false;

	if (getBufferSize()<=i32Arg && !resizeBuffer(i32Arg+1))
		return false;

	getBuffer()[i32Arg]=u8Arg;

	return false;

}

bool D2CBuffer::fillBuffer(UI8 u8Arg,I32 i32Arg1,I32 i32Arg2) {

	if (i32Arg1<0 || i32Arg1>i32Arg2)
		return false;

	if (getBufferSize()<=i32Arg2 && !resizeBuffer(i32Arg2+1))
		return false;

	FillMemory(getBuffer()+i32Arg1,i32Arg2-i32Arg1+1,u8Arg);

	return true;

}

bool D2CBuffer::appendBuffer(const BUF bfArg,I32 i32Arg) {

	if (bfArg==NULL || i32Arg<1)
		return false;

	I32 i32Size=getBufferSize();

	if (!resizeBuffer(getBufferSize()+i32Arg,false))
		return false;

	memcpy(getBuffer()+i32Size,bfArg,i32Arg);

	return true;

};

bool D2CBuffer::appendBuffer(const D2IBufferReader *pbArg) {

	if (pbArg==NULL)
		return false;

	BUF bfTmp=pbArg->copyBuffer();

	bool bTmp=appendBuffer(bfTmp,pbArg->getBufferSize());

	delete [] bfTmp;

	return bTmp;

}

bool D2CBuffer::appendBufferByte(UI8 u8Arg) {

	if (!resizeBuffer(getBufferSize()+1))
		return false;

	getBuffer()[getBufferSize()-1]=u8Arg;

	return true;

}

D2CBuffer::~D2CBuffer() {

	deleteBuffer();

}
