#include "windef.h"
#include "define.h"

class D2IObject {

public:

	virtual UI32 getObjectType() const=0;
	virtual UI8 getVersion() const=0;
	virtual UI8 getDataType() const=0;
	virtual UI16 getEncoding() const=0;
	virtual void * getInterface(UI32) const=0;
	virtual CSTR getCSTR()=0;

	virtual ~D2IObject() {}

};

class D2IBufferReader {

public:

	virtual BUF copyBuffer() const=0;
	virtual I32 getBufferSize() const=0;
	virtual UI8 readBufferByte(I32) const=0;
	virtual bool equalBuffer(const BUF,I32) const=0;
	virtual bool equalBuffer(const D2IBufferReader *) const=0;

	~D2IBufferReader() {}

};

class D2IBufferWriter {

public:

	virtual BUF getBuffer() const=0;
	virtual bool resizeBuffer(I32,bool bArg=true)=0;
	virtual bool assignBuffer(const BUF,I32)=0;
	virtual bool assignBuffer(const D2IBufferReader *)=0;
	virtual bool writeBuffer(const BUF,I32,I32)=0;
	virtual bool writeBufferByte(UI8,I32)=0;
	virtual bool fillBuffer(UI8,I32,I32)=0;
	virtual bool appendBuffer(const BUF,I32)=0;
	virtual bool appendBuffer(const D2IBufferReader *)=0;
	virtual bool appendBufferByte(UI8)=0;

	~D2IBufferWriter() {}

};

class D2IBufferedObject:public D2IObject,public D2IBufferReader,public D2IBufferWriter {

public:

	virtual ~D2IBufferedObject() {}

};

class D2CBuffer:public D2IBufferedObject {

public:

	D2CBuffer();
	D2CBuffer(const BUF,I32);
	D2CBuffer(const D2IBufferReader *);

	virtual bool clear();

	virtual ~D2CBuffer();

	// D2IObject

	virtual UI32 getObjectType() const;
	virtual UI8 getVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32) const;
	virtual CSTR getCSTR();

	// D2IBufferReader

	virtual BUF copyBuffer() const;
	virtual I32 getBufferSize() const;
	virtual UI8 readBufferByte(I32) const;
	virtual bool equalBuffer(const BUF,I32) const;
	virtual bool equalBuffer(const D2IBufferReader *) const;

	// D2IBufferWriter

	virtual BUF getBuffer() const;
	virtual bool resizeBuffer(I32,bool bArg=true);
	virtual bool assignBuffer(const BUF,I32);
	virtual bool assignBuffer(const D2IBufferReader *);
	virtual bool writeBuffer(const BUF,I32,I32);
	virtual bool writeBufferByte(UI8,I32);
	virtual bool fillBuffer(UI8,I32,I32);
	virtual bool appendBuffer(const BUF,I32);
	virtual bool appendBuffer(const D2IBufferReader *);
	virtual bool appendBufferByte(UI8);

protected:

	BUF m_bfBuffer;
	CSTR m_csStr;
	I32 m_i32BufferSize;

	virtual bool setBuffer(const BUF);
	virtual bool setBufferSize(I32);
	virtual bool deleteBuffer();

};

class D2CFile {

public:

	D2CFile();
	D2CFile(CSTR);

	CSTR getFilePath() const;
	CSTR getFileName() const;
	CSTR getFileDir() const;
	I32 getFileSize() const;

	bool open(UI32);
	bool open(CSTR,UI32);

	D2IBufferReader * read();
	UI8 readByte(I32);

	bool write(D2IBufferReader *);
	bool writeByte(UI8,I32);

	bool append(D2IBufferReader *);
	bool appendByte(UI8);

	bool close();

	~D2CFile();

private:

	FILE *m_fp;
	CHR m_csPath[256],m_csName[256],m_csDir[256];
	bool m_bWritable;

	bool reset();

};
